==> skyfish-0.60.tar.gz: skyfish-0.60/module/drv.h <==
/*-
 * Copyright (c) 2008 Andrey V. Surkov <nsand@sura.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: drv.h,v 0.52 2008/05/23 07:54:36 nsand Exp $
 */
/*
       /dev/skyfish
*/
#include <sys/conf.h>   /* cdevsw struct */
#include <sys/uio.h>    /* uio struct */
#include <sys/malloc.h>
#include <sys/kthread.h>

#ifndef _DRV_SKYFISH_
#define	_DRV_SKYFISH_

/* Function prototypes */
static d_open_t      echo_open;
static d_close_t     echo_close;
static d_read_t      echo_read;
static d_kqfilter_t  echo_kqfilter;

/* Character device entry points */
static struct cdevsw echo_cdevsw = {
       .d_version = D_VERSION,
       .d_open = echo_open,
       .d_close = echo_close,
       .d_read = echo_read,
       .d_kqfilter = echo_kqfilter,
       .d_name = "skyfish"
};

MALLOC_DECLARE(M_FIFOBUF);
MALLOC_DEFINE(M_ECHOBUF, "skyfishbuffer", "buffer for skyfish module");


static int filt_read(struct knote *kn, long hint);
static int filt_detach(struct knote *kn, long hint);
static struct cdev *echo_dev;
//static u_long isdata;				// -   
//static struct ipdata *echomsg;			
static struct selinfo echo_rsel;		//  kevent/kqueue
static struct filterops echoread_filtops =
	{ 1, 
	NULL, 
	filt_detach, 
	filt_read};



static int
echo_loader(struct module *m, int what, void *arg)
{
       int err = 0;

       switch (what) {
       case MOD_LOAD:                /* kldload */
               echo_dev = make_dev(&echo_cdevsw,
                   0, UID_ROOT,GID_WHEEL, 0666, "skyfish");
               /* kmalloc memory for use by this driver */
               printf("skyfish device loaded.\n");
		isdata=0; //threshold=0; cntw=0; cntr=0;
		//cntread=0;
		rcvdata = 0;
		fifobuf = malloc(BUFFERSIZE,M_DEVBUF, M_NOWAIT);
//		tmpbuf = malloc(TMPBUF,M_DEVBUF, M_NOWAIT);
		if (fifobuf == NULL) {
			    printf("no memory for buffer\n");
			    return -1;}
		knlist_init(&echo_rsel.si_note, NULL, NULL, NULL, NULL);
                      break;
       case MOD_UNLOAD:
               destroy_dev(echo_dev);
	       knlist_destroy(&echo_rsel.si_note);
               free(fifobuf, M_DEVBUF);
//	       free(tmpbuf, M_DEVBUF);
               printf("skyfish device unloaded.\n");
               break;
       default:
               err = EINVAL;
               break;
       }
       return(err);
}


static int
echo_open(struct cdev *dev, int oflags, int devtype, struct thread *p)
{
    return(0);
}

static int
echo_close(struct cdev *dev, int fflag, int devtype, struct thread *p)
{
    return(0);
}

static int
echo_kqfilter(struct cdev *dev, struct knote *kn)
{
	switch (kn->kn_filter) {
	case EVFILT_READ:
		kn->kn_fop = &echoread_filtops;
		printf("echo_kqfilter: knlist_add  isdata=%lu\n",isdata);
		rcvdata = 0;
		rcvdata2 = 0;
		pos_r=0; pos_w=0; isdata=0;
		
		break;
	case EVFILT_WRITE:
		break;
	default:
		return (EINVAL);
	}
    
    kn->kn_hook = (caddr_t)dev;		
    knlist_add(&echo_rsel.si_note, kn, 0);
    return (0);
}

static int
echo_read(struct cdev *dev, struct uio *uio, int ioflag)
{
    critical_enter();
    int err = 0;
    u_char *data;
    u_long *pl;
    
    if (pos_r == pos_w) {
	isdata--;
	critical_exit();
	return 1;
    }
    
    pl = (u_long*)(fifobuf + pos_r);
    u_long len = *pl;
    pos_f = pos_r + len + 4; 
    if (pos_f >= (BUFFERSIZE - 4)) {
	pos_r = 0;
    }
    
    data = (u_char*)(fifobuf + pos_r + 4);
    pos_r += len + 4;
    if (isdata == 0) printf("DATA ------!\n");
    isdata--; 
	if ((err = uiomove(data,len,uio)) != 0) {
    	    printf("uiomove failed!\n");
	    }

    critical_exit();
return err;
}

static int
filt_read(struct knote *kn, long hint)
{
	critical_enter();
	int	 ret;
	if ((kn->kn_data = isdata) > 0) {	//  - . 
		ret = 1;
	} else {
		ret = 0;
	}
	critical_exit();
	return (ret);
}

static int
filt_detach(struct knote *kn, long hint)
{
    knlist_remove(&echo_rsel.si_note,kn,0);
    printf("echo_kqfilter: knlist_remove\n");
    printf("rcv_data=%lu  rcvdata2=%lu\tisdata=%lu pos_r=%lu pos_w=%lu\n",rcvdata,rcvdata2,isdata,pos_r,pos_w);
    return 0;
}

DEV_MODULE(echo,echo_loader,NULL);

#endif /* !_DRV_SKYFISH_ */

==> skyfish-0.83.tar.gz: skyfish-0.83/module/drv.h <==
/*-
 * Copyright (c) 2008 Andrey V. Surkov <nsand@sura.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: drv.h,v 0.52 2008/05/23 07:54:36 nsand Exp $
 */

/*
*    driver fo pseudo device /dev/skyfish
*/
#include <sys/conf.h>   /* cdevsw struct */
#include <sys/uio.h>    /* uio struct */
#include <sys/malloc.h>
#include <sys/kthread.h>

#ifndef _DRV_SKYFISH_
#define	_DRV_SKYFISH_

/* Function prototypes */
static d_open_t      echo_open;
static d_close_t     echo_close;
static d_read_t      echo_read;
static d_kqfilter_t  echo_kqfilter;

/* Character device entry points */
static struct cdevsw echo_cdevsw = {
       .d_version = D_VERSION,
       .d_open = echo_open,
       .d_close = echo_close,
       .d_read = echo_read,
       .d_kqfilter = echo_kqfilter,
       .d_name = "skyfish"
};

MALLOC_DECLARE(M_FIFOBUF);
MALLOC_DEFINE(M_ECHOBUF, "skyfishbuffer", "buffer for skyfish module");


static int filt_read(struct knote *kn, long hint);
static void filt_kqdetach(struct knote *kn);
static struct cdev *echo_dev;
static struct selinfo echo_rsel;		/* for kevent/kqueue */


static struct filterops echoread_filtops = {
        .f_isfd =       1,
        .f_attach =     NULL,
        .f_detach =     filt_kqdetach,
        .f_event =      filt_read,
};        

static int socketlock;	/* only one may be open to device */

static int
echo_loader(struct module *m, int what, void *arg)
{
       int err = 0;

       switch (what) {
       case MOD_LOAD:                /* kldload */
               echo_dev = make_dev(&echo_cdevsw,
                   0, UID_ROOT,GID_WHEEL, 0666, "skyfish");
               /* kmalloc memory for use by this driver */
               printf("skyfish device loaded.\n");
		isdata=0;
		rcvdata = 0;
		fifobuf = malloc(BUFFERSIZE,M_DEVBUF, M_NOWAIT);
		if (fifobuf == NULL) {
			    printf("no memory for buffer\n");
			    return -1;}
		knlist_init(&echo_rsel.si_note, NULL, NULL, NULL, NULL);
		socketlock = 0;
                      break;
       case MOD_UNLOAD:
               destroy_dev(echo_dev);
	       knlist_destroy(&echo_rsel.si_note);
               free(fifobuf, M_DEVBUF);
               printf("skyfish device unloaded.\n");
               break;
       default:
               err = EINVAL;
               break;
       }
       return(err);
}


static int
echo_open(struct cdev *dev, int oflags, int devtype, struct thread *p)
{
    if (socketlock == 0) {socketlock = 1; return 0;}
    return 1;
}

static int
echo_close(struct cdev *dev, int fflag, int devtype, struct thread *p)
{
    if (socketlock == 1) {socketlock = 0; return 0;}
    return 1;
}

static int
echo_kqfilter(struct cdev *dev, struct knote *kn)
{
	switch (kn->kn_filter) {
	case EVFILT_READ:
		kn->kn_fop = &echoread_filtops;
//		printf("echo_kqfilter: knlist_add  isdata=%lu\n",isdata);
		rcvdata = 0;
		rcvdata2 = 0;
		pos_r=0; pos_w=0; isdata=0;
		
		break;
	case EVFILT_WRITE:
		break;
	default:
		return (EINVAL);
	}	
    
    kn->kn_hook = (caddr_t)dev;		
    knlist_add(&echo_rsel.si_note, kn, 0);
    return (0);
}

static int
echo_read(struct cdev *dev, struct uio *uio, int ioflag)
{
    critical_enter();
    int err = 0;
    u_char *data;
    uint32_t *pl;
    
    if (pos_r == pos_w) {
	isdata--;
	critical_exit();
	return 1;
    }
    
    pl = (uint32_t*)(fifobuf + pos_r);
    uint32_t len = *pl;
    pos_f = pos_r + len + 4; 
    if (pos_f >= (BUFFERSIZE - 4)) {
	pos_r = 0;
    }
    
    data = (u_char*)(fifobuf + pos_r + 4);
    pos_r += len + 4;
    if (isdata == 0) printf("DATA ------!\n");
    isdata--; 
	if ((err = uiomove(data,len,uio)) != 0) {
    	    printf("uiomove failed!\n");
	    }

    critical_exit();
return err;
}

static int
filt_read(struct knote *kn, long hint)
{
	critical_enter();
	int	 ret;
	if ((kn->kn_data = isdata) > 0) {	//  - . 
		ret = 1;
	} else {
		ret = 0;
	}
	critical_exit();
	return (ret);
}

static void
filt_kqdetach(struct knote *kn)
{
    knlist_remove(&echo_rsel.si_note,kn,0);
  //  printf("echo_kqfilter: knlist_remove\n");
  //  printf("rcv_data=%lu  rcvdata2=%lu\tisdata=%lu pos_r=%lu pos_w=%lu\n",rcvdata,rcvdata2,isdata,pos_r,pos_w);
};

DEV_MODULE(echo,echo_loader,NULL);

#endif /* !_DRV_SKYFISH_ */
